@@ -26,21 +26,24 @@
 #include <QObject>
 #include <QPointer>
 
+#include "konsoleprivate_export.h"
 // Konsole
-#include "Profile.h"
 
 class KActionCollection;
 class KConfigGroup;
 
 namespace Konsole {
+
 class ColorScheme;
+class Profile;
 class Session;
-class TerminalDisplay;
-class TabbedViewContainer;
 class SessionController;
+class TabbedViewContainer;
+class TabbedViewContainer;
+class TerminalDisplay;
 class ViewProperties;
 class ViewSplitter;
-class TabbedViewContainer;
+class SessionDisplayConnection;
 
 /**
  * Manages the terminal display widgets in a Konsole window or part.
@@ -84,7 +87,7 @@ public:
      * Applies the view-specific settings associated with specified @p profile
      * to the terminal display @p view.
      */
-    void applyProfileToView(TerminalDisplay *view, const Profile::Ptr &profile);
+    void applyProfileToView(TerminalDisplay *view, const QExplicitlySharedDataPointer<Profile> &profile);
 
     void toggleActionsBasedOnState();
     /**
@@ -174,15 +177,12 @@ public:
     int managerId() const;
 
     /** Returns a list of sessions in this ViewManager */
-    QList<Session *> sessions()
-    {
-        return _sessionMap.values();
-    }
+    QList<Session *> sessions();
 
     /**
      * Returns whether the @p profile has the blur setting enabled
      */
-    static bool profileHasBlurEnabled(const Profile::Ptr &profile);
+    static bool profileHasBlurEnabled(const QExplicitlySharedDataPointer<Profile> &profile);
 
     /** returns the active tab from the view
     */
@@ -190,7 +190,7 @@ public:
     TerminalDisplay *createView(Session *session);
     void attachView(TerminalDisplay *terminal, Session *session);
 
-    static const ColorScheme *colorSchemeForProfile(const Profile::Ptr &profile);
+    static const ColorScheme *colorSchemeForProfile(const QExplicitlySharedDataPointer<Profile> &profile);
     /** Reorder the terminal display history list */
     void updateTerminalDisplayHistory(TerminalDisplay *terminalDisplay = nullptr, bool remove = false);
 
@@ -203,7 +203,7 @@ public:
      * The session has specified @p profile, working @p directory
      * and configured environment.
      */
-    Session* createSession(const Profile::Ptr &profile, const QString &directory = QString());
+    Session* createSession(const QExplicitlySharedDataPointer<Profile> &profile, const QString &directory = QString());
 
 Q_SIGNALS:
     /** Emitted when the last view is removed from the view manager */
@@ -243,7 +243,7 @@ Q_SIGNALS:
     /** Requests creation of a new view with the default profile. */
     void newViewRequest();
     /** Requests creation of a new view, with the selected profile. */
-    void newViewWithProfileRequest(const Profile::Ptr&);
+    void newViewWithProfileRequest(const QExplicitlySharedDataPointer<Profile> &profile);
 
 public Q_SLOTS:
     /** DBus slot that returns the number of sessions in the current view. */
@@ -251,7 +251,7 @@ public Q_SLOTS:
 
     /**
      * DBus slot that returns the unique ids of the sessions in the
-     * current view.  The returned list is not sorted.
+     * current view.  The returned list is ordered by tab.
      * QList<int> is not printable by qdbus so we use QStringList
      */
     Q_SCRIPTABLE QStringList sessionList();
@@ -370,7 +370,7 @@ private Q_SLOTS:
     void containerViewsChanged(TabbedViewContainer *container);
 
     // called when a profile changes
-    void profileChanged(const Profile::Ptr &profile);
+    void profileChanged(const QExplicitlySharedDataPointer<Profile> &profile);
 
     void updateViewsForSession(Session *session);
 
@@ -429,7 +429,9 @@ private:
     QPointer<TabbedViewContainer> _viewContainer;
     QPointer<SessionController> _pluggedController;
 
-    QHash<TerminalDisplay *, Session *> _sessionMap;
+    // TODO: This map must be separated into a new class containing the global link map between sessions and displays used by the entire konsole
+    // The solution implemented so far is initial and should be reviewed
+    QVector<SessionDisplayConnection*> _sessionMap;
 
     KActionCollection *_actionCollection;
 
